<?php

        /*
         * API (Application Programming Interface) für Pegelonline Webservice Version 2.4.
         *
         * Vereinfacht Zugriff auf die einzelnen Abfragen.
         *
         * Über den unterhalb der Klasse aufgeführten Code können alle Abfragen als PHP Script
         * ausgeführt und getestet werden.
         *
         * Die Klasse kann über "include '<Pfad>'" in eine PHP-Webpage eingebunden werden.
         *
         * Vorraussetzungen:
         *              - mindestens PHP Version 5.1
         *              - PHP Soap Extension installiert (z.B. yum info php-soap.x86_64 unter RHEL/CentOS)
         *
         * Link: http://www.pegelonline.wsv.de/webservice/aktuell.jsp
         */
        class PegelonlineWebserviceClientApiVersion2_4 {

                var $soapClient;
                var $localTimezone;

                /*
                 * Konstruktur
                 *
                 * $trace: Wenn TRUE kann der Soap Request und Response als XML angezeigt werden
                 * $exceptions: Wenn TRUE wird bei einem Soap-Fehler eine Exception vom Typ SoapFault geworfen wird (TRUE empfehlenswert)
                 */
                function PegelonlineWebserviceClientApiVersion2_4($trace = TRUE, $exceptions = TRUE) {
                        $this->soapClient = new SoapClient(
                                'http://pegelonline.wsv.de/webservices/version2_4/2009/05/12/PegelonlineWebservice?WSDL',
                                array ( 'trace'   => $trace, 'exceptions'=> $exceptions));

                        //verwendet in toDateTime
                        $this->localTimezone = new DateTimeZone("Europe/Berlin");
                }

                /*
                 * Einzelne Abfragen von Pegelonline-Webservice.
                 *
                 * Eine sprachenagnostische Dokumentation findet sich unter
                 * http://www.pegelonline.wsv.de/webservice/dokuAkt.jsp
                 */

                function getParameterList() {
                        return $this->modifyArrayResponse($this->soapClient->getParameterList()->getParameterListReturn);
                }

                function getGewaesserList($parameterName) {
                        $params = array (
                                'parameterName'=>$parameterName
                        );

                        return $this->modifyArrayResponse($this->soapClient->getGewaesserList($params)->getGewaesserListReturn);
                }

                function getGewaesser($gewaesserName) {
                        $params = array (
                                'gewaesserName'=>$gewaesserName
                        );
                        $this->soapClient->getGewaesser($params)->getGewaesserReturn;
                }

                function getMessstellenList($parameterName, $gewaesserNamen) {
                        $params = array (
                                'parameterName'=>$parameterName,
                                'gewaesserNamen'=>$gewaesserNamen
                        );
                        return $this->modifyArrayResponse($this->soapClient->getMessstellenList($params)->getMessstellenListReturn);
                }

                function getMessstelle($messstelleNummer, $messstelleName) {
                        $params = array (
                                'messstelleNummer'=>$messstelleNummer,
                                'messstelleName'=>$messstelleName
                        );
                        return $this->soapClient->getMessstelle($params)->getMessstelleReturn;
                }

                function getMessstellenParameter($messstelleNummer, $messstelleName) {
                        $params = array (
                                'messstelleNummer'=>$messstelleNummer,
                                'messstelleName'=>$messstelleName
                        );
                        return $this->modifyArrayResponse($this->soapClient->getMessstellenParameter($params)->getMessstellenParameterReturn);
                }

                function getDatenverfuegbarkeit($parameterName, $messstellenNummern, $messstellenNamen) {
                        $params = array (
                                'parameterName'=>$parameterName,
                                'messstellenNummern'=>$messstellenNummern,
                                        'messstellenNamen'=>$messstellenNamen,
                        );
                        return $this->soapClient->getDatenverfuegbarkeit($params)->getDatenverfuegbarkeitReturn;
                }

                function getPegelinformationen($parameterName, $messstellenNummern, $messstellenNamen) {
                        $params = array (
                                'parameterName'=>$parameterName,
                                'messstellenNummern'=>$messstellenNummern,
                                        'messstellenNamen'=>$messstellenNamen,
                        );
                        return $this->modifyArrayResponse($this->soapClient->getPegelinformationen($params)->getPegelinformationenReturn);
                }

                /*
                 * $start: DateTime-String in Format akzeptiert durch strtotime()
                 * $ende:  DateTime-String in Format akzeptiert durch strtotime()
                 */
                function getGanglinienUrl($parameterName, $messstellenNummern, $messstellenNamen, $start, $ende, $einzelansicht, $design) {
                        $params = array (
                                'parameterName'=>$parameterName,
                                'messstellenNummern'=>$messstellenNummern,
                                        'messstellenNamen'=>$messstellenNamen,
                                'start'=>empty($start) ? null : $this->toW3CDateTime($start), //Konvertierung in W3C-Format (unix-timestamp als integer würde allerdings auch direkt gehen)
                                'ende'=>empty($ende) ? null : $this->toW3CDateTime($ende), //dito
                                'einzelansicht'=>$einzelansicht,
                                'design'=>$design
                        );

                        return $this->soapClient->getGanglinienUrl($params)->getGanglinienUrlReturn;
                }

                /*
                 * $start: DateTime-String in Format akzeptiert durch strtotime()
                 * $ende:  DateTime-String in Format akzeptiert durch strtotime()
                 */
                function getGanglinienImage($parameterName, $messstellenNummern, $messstellenNamen, $start, $ende, $laenge, $hoehe) {
                        $params = array (
                                'parameterName'=>$parameterName,
                                'messstellenNummern'=>$messstellenNummern,
                                        'messstellenNamen'=>$messstellenNamen,
                                'start'=>empty($start) ? null : $this->toW3CDateTime($start), //Konvertierung in W3C-Format (unix-timestamp als integer würde allerdings auch direkt gehen)
                                'ende'=>empty($ende) ? null : $this->toW3CDateTime($ende), //dito
                                'laenge'=>$laenge,
                                'hoehe'=>$hoehe
                        );

                        return $this->soapClient->getGanglinienImage($params)->getGanglinienImageReturn;
                }

                /*
                 * $start: DateTime-String in Format akzeptiert durch strtotime()
                 * $ende:  DateTime-String in Format akzeptiert durch strtotime()
                 */
                function getMessungenDatei($parameterName, $messstellenNummern, $messstellenNamen, $start, $ende) {
                        $params = array (
                                'parameterName'=>$parameterName,
                                'messstellenNummern'=>$messstellenNummern,
                                        'messstellenNamen'=>$messstellenNamen,
                                'start'=>empty($start) ? null : $this->toW3CDateTime($start), //Konvertierung in W3C-Format (unix-timestamp als integer würde allerdings auch direkt gehen)
                                'ende'=>empty($ende) ? null : $this->toW3CDateTime($ende), //dito
                        );

                        return $this->soapClient->getMessungenDatei($params)->getMessungenDateiReturn;
                }

                /*
                 * Hilfsfunktion zur Erstellung eines W3C-konformen (ISO-8601-Standard)
                 * Datums- und Zeitausdrucks aus dem Format 'd.m.Y'.
                 *
                 * siehe:
                 *              - http://www.w3.org/TR/xmlschema-2/#dateTime
                 *              - http://www.cl.cam.ac.uk/~mgk25/iso-time.html
                 *
                 * $dateTime: DateTime-String in Format akzeptiert durch strtotime()
                 */
                function toW3CDateTime($strDateTime) {
                        $dateTime = new DateTime($strDateTime, $this->localTimezone);
                        return $dateTime->format(DATE_W3C);
                }

                /*
                 * Hilfsfunktion, um aus einem W3C-konformem Datums- und Zeitausdrucks
                 * eine beliebige Datums-/Zeitdarstellung für die lokale Zeitzone zu generieren
                 *
                 * $format: Beliebiges Datumsformat der Funktion strftime
                 * $w3cDateTime: Nach W3C-Standard formatierter Datums-/Zeitausdruck als string
                 */
                function formatToDateTime($format, $w3cDateTime) {
                        return $this->toDateTime($w3cDateTime)->format($format);
                }

                /*
                 * Hilfsfunktion, um aus einem W3C-konformem Datums- und Zeitausdrucks
                 * eine DateTime-Objekt für die lokale Zeitzone zu generieren
                 *
                 * $w3cDateTime: Nach W3C-Standard formatierter Datums-/Zeitausdruck als string
                 */
                function toDateTime($w3cDateTime){
                        $date = new DateTime($w3cDateTime, $this->localTimezone);
                        $date->setTimezone($this->localTimezone); //siehe PHP-Bug: #40743
                        return $date;
                }


                /*
                 * Modifiziert den Response und kompensiert damit die ungünstige 'auto-magic'
                 * Erstellung der Proxy-Objekte bei bestimmten Soap Responses durch Php Soap Extension
                 * (bei 0 und 1 Elementen innerhalb eines Arrays im Soap Response)
                 *
                 * Somit kann der response immer in einer foreach-Schleife verwendet werden
                 *
                 * Sollte bei allen Abfragen, die arrays liefern aufgerufen werden
                 */
                function &modifyArrayResponse(&$response) {
                    //Soap Extension liefert Null bei leerer Ergebnismenge
                    if ($response == NULL) {
                        $emptyArray = array();
                        return $emptyArray;

                        //Soap Extension liefert kein array mit einem Element, wenn nur ein Element durch Webservice
                        //zurück gegeben wird
                    } elseif (!is_array($response)) {
                        return array($response);

                        //Ansonsten ok
                    } else {
                        return $response;
                    }
                }

                /*
                 * Anzeigen des Soap XML Requests. Funktioniert nur, wenn
                 * die trace-Option im Konstruktor aktiviert wurde.
                 */
                function echoLastSoapRequest() {
                        echo "Last SOAP Request:\n";
                        echo $this->soapClient->__getLastRequest() . "\n";
                }

                /*
                 * Anzeigen des Soap XML Response. Funktioniert nur, wenn
                 * die trace-Option im Konstruktor aktiviert wurde.
                 */
                function echoLastSoapResponse() {
                        echo "Last SOAP Response:\n";
                        echo $this->soapClient->__getLastResponse() . "\n";
                }
        }

        /*
         * Illustriert den Aufruf aller Abfragen von Pegelonline Webservice Aktuell.
         *
         * Der Graph der Proxy-Objekte der PHP Soap Extension wird über var_dump(...)
         * jeweils ausgegeben.
         *
         * Beispielhaft wird in der Abfrage 'getPegelinformation' auf die Proxy-Objekte
         * zugegriffen. Dies illustriert den Zusammenhang der über var_dump(...) ausgegebenen
         * Darstellung und dem entsprechenden Funktionsaufruf für einzelne Felder der Datentypen
         * des Webservices.
         * Dort wird auch die Hilfsfunktion toDateTime verwendet, um aus einem W3C-konformen Datumsausdruck
         * einen beliebigen Datumsstring zu generieren.
         */
        try {

                $clientApi = new PegelonlineWebserviceClientApiVersion2_4();

                //echo "Abfrage getParameterList:\n";
                //$result = $clientApi->getParameterList();
                //var_dump($result);

                echo "Abfrage getGewaesserList:\n";
                $result = $clientApi->getGewaesserList('WASSERSTAND ROHDATEN');
                var_dump($result);

                //echo "Abfrage getGewaesser:\n";
                //$result = $clientApi->getGewaesser('ALLER');
                //var_dump($result);

                //echo "Abfrage getMessstellenList :\n";
                //$result = $clientApi->getMessstellenList(NULL, array('RHEIN', 'MAIN'));
                //var_dump($result);

                //echo "Abfrage getMessstelle:\n";
                //$result = $clientApi->getMessstelle(NULL, 'TRUNSTADT');
                //var_dump($result);

                //echo "Abfrage getMessstellenParameter :\n";
                //$result = $clientApi->getMessstellenParameter(NULL, 'TRUNSTADT');
                //var_dump($result);

                //echo "Abfrage getDatenverfuegbarkeit:\n";
                //$result = $clientApi->getDatenverfuegbarkeit('WASSERSTAND ROHDATEN', NULL, array('DRESDEN', 'RUHRORT'));
                //var_dump($result);

/*
                echo "Abfrage getPegelinformationen:\n";
                $result = $clientApi->getPegelinformationen("WASSERSTAND ROHDATEN", NULL, array('TRUNSTADT', 'DRESDEN', 'KAUB'));
                foreach ($result as $index => $element) {
                        echo $index . ". ArrayElement\n";
                        echo "Gewaesser :" . $element->pegelonlineGewaesser->name . "\n";
                        echo "Messstelle :" . $element->pegelonlineMessstelle->name . "\n";
                        echo "Geografische Koordination :" . $element->pegelonlineKoordinate->rechtswert . ", " . $element->pegelonlineKoordinate->hochwert . "\n";
                        echo
                                "Aktuelle Messung: " .
                                $clientApi->formatToDateTime('d.m.Y H:i I', $element->pegelonlineAktuelleMessung->zeitpunkt) . "  " .
                                $element->pegelonlineAktuelleMessung->messwert . "\n";
                        echo "\n";

                        //Vorhersagen
                        if (isset($element->pegelonlineVorhersagen->pegelonlineVorhersagen)) {
                                foreach ($element->pegelonlineVorhersagen->pegelonlineVorhersagen as $index2 => $element2) {
                                        echo "Vorhersage: " . $clientApi->formatToDateTime('d.m.Y H:i I', $element2->zeitpunkt) . ", Wert: " . $element2->wert . ", Typ: " . $element2->type . "\n";
                                }
                        }
                }
                echo "\n";
                var_dump($result);
*/
                //echo "Abfrage getGanglinienUrl:\n";
                //$result = $clientApi->getGanglinienUrl(NULL, NULL, array('TRUNSTADT', 'DRESDEN'), "-10 day", "-2 day", TRUE, NULL);
                //echo $result . "\n\n";
                //var_dump($result);

/*
                echo "Abfrage getGanglinienImage:\n";
                $result = $clientApi->getGanglinienImage(NULL, NULL, array('TRUNSTADT', 'DRESDEN'), "-11 day", "-2 day", 600, 400);
                $file = fopen("test.gif", "w+");
                fwrite($file, $result);
                fclose($file);
                // var_dump($result);
*/
/*
                echo "Abfrage getMessungenDatei:\n";
                $result = $clientApi->getMessungenDatei('WASSERSTAND ROHDATEN', NULL, array('TRUNSTADT', 'DRESDEN'), "-10 day", "-2 day");
                $file = fopen("test.zip", "w+");
                fwrite($file, $result);
                fclose($file);
                // var_dump($result);
*/
                //$clientApi->echoLastSoapRequest();
                //$clientApi->echoLastSoapResponse();

        } catch (SoapFault $fault) {
                var_dump($fault);
        }
